const WebSocket = require('ws');
const webSocketHost = "localhost"
const webSocketPort = 9898
const wss = new WebSocket.Server({ host: webSocketHost, port: webSocketPort });
const http = require("http");

var PROTO_PATH = __dirname + '/../protos/v3/projector.proto';

var grpc = require('@grpc/grpc-js');
var protoLoader = require('@grpc/proto-loader');
var packageDefinition = protoLoader.loadSync(
  PROTO_PATH,
  {
    keepCase: true,
    longs: String,
    enums: String,
    defaults: true,
    oneofs: true
  });

var client = null;

function main() {
  var target = '127.0.0.1:50051';
  let pkg = grpc.loadPackageDefinition(packageDefinition);
  let projector = pkg.provee["Projector"];
  client = new projector(target, grpc.credentials.createInsecure());
  console.log(`Connected to grpc Server at: ${target}`)
}

main();

wss.on('connection', function connection(ws) {
  console.log(`Client connected with websocket`);

  ws.on('message', function incoming(message) {
    parseMessage(ws, message);
  });
});

function parseMessage(ws, message) {
  const jsonMessage = JSON.parse(message);

  switch (jsonMessage["type"]) {
    case "requestPointStream":
      console.log("Requested stream");
      getPointStream(ws);
      break;
    case "sendDataRow":
      console.log(jsonMessage);
      echoPoint(ws, jsonMessage["row"]);
      break;
    default:
      console.log("Error! Unknown request:" + jsonMessage["type"]);
  }
}


function echoPoint(ws, dataRow) {
  console.log(dataRow);
  ws.send(JSON.stringify({
      "id": dataRow[0]
      , "x": dataRow[1],
      "y": dataRow[2]
    
  }));
}

function getPointStream(ws) {
  console.log("stream")
  var trainingSetRow = {id:"",hdvector:null};
  var stream = client.getProjectionPoints(trainingSetRow);

  stream.on("data", (response) => {
    sendDataStream(JSON.stringify(response),ws)
  });
  
  stream.on("error", (err) => {  
    console.log(
      `Unexpected stream error: code = ${err.code}` +
      `, message = "${err.message}"`
    );
  });
}

function sendDataStream(data, ws) {
  var list = data.toString().trim().split("\n")

  for (var i = 0; i < list.length; i++) {
    try {
      var response = list[i];
      ws.send(response);

    } catch (e) {
      console.log(e);
    }
  }
}


